@@ -22,7 +22,7 @@
 #define TTFAUTOHINT_STRING "; ttfautohint"
 #define TTFAUTOHINT_STRING_WIDE "\0;\0 \0t\0t\0f\0a\0u\0t\0o\0h\0i\0n\0t"
 
-// build string which gets appended to the `Version' field(s)
+// build string that gets appended to the `Version' field(s)
 
 extern "C" {
 
@@ -63,6 +63,7 @@ build_version_string(Info_Data* idata)
   d += sprintf(d, " -r %d", idata->hinting_range_max);
   d += sprintf(d, " -G %d", idata->hinting_limit);
   d += sprintf(d, " -x %d", idata->increase_x_height);
+  d += sprintf(d, " -f %s", idata->fallback_script);
 
   count = 0;
   strong[0] = '\0';
@@ -75,16 +76,17 @@ build_version_string(Info_Data* idata)
     strong[count++] = 'G';
   if (idata->dw_cleartype_strong_stem_width)
     strong[count++] = 'D';
-  d += sprintf(d, " -w \"%s\"", strong);
+  if (*strong)
+    d += sprintf(d, " -w %s", strong);
+  else
+    d += sprintf(d, " -w \"\"");
 
   if (idata->windows_compatibility)
     d += sprintf(d, " -W");
   if (idata->pre_hinting)
     d += sprintf(d, " -p");
-  if (!idata->hint_composites)
+  if (idata->hint_composites)
     d += sprintf(d, " -c");
-  if (idata->latin_fallback)
-    d += sprintf(d, " -f");
   if (idata->symbol)
     d += sprintf(d, " -s");
   if (idata->x_height_snapping_exceptions)
