@@ -45,13 +45,16 @@ class Main_GUI
 
 public:
   Main_GUI(int, int, int,
+           bool, bool,
+           bool, int,
+           const char*,
            bool, bool, bool,
-           int, const char*,
-           bool, bool, bool,
-           bool, bool, int, bool,
+           bool, bool, const char*, bool,
            bool);
   ~Main_GUI();
 
+  void create_alternative_layout();
+
 protected:
   void closeEvent(QCloseEvent*);
 
@@ -87,7 +90,7 @@ private:
   int pre_hinting;
   int hint_composites;
   int no_info;
-  int latin_fallback;
+  int fallback_script_idx;
   int symbol;
   int dehint;
 
@@ -108,9 +111,11 @@ private:
   QMenu* file_menu;
   QMenu* help_menu;
 
+  QLabel* input_label;
   Drag_Drop_Line_Edit* input_line;
   QPushButton* input_button;
 
+  QLabel* output_label;
   Drag_Drop_Line_Edit* output_line;
   QPushButton* output_button;
 
