@@ -138,6 +138,17 @@ typedef struct glyf_Data_
   FT_ULong cvt_idx;
   FT_ULong fpgm_idx;
   FT_ULong prep_idx;
+
+  /* scripts present in a font get a running number */
+  FT_UInt script_ids[TA_SCRIPT_MAX];
+  FT_UInt num_used_scripts;
+
+  /* we have separate CVT data for each script */
+  FT_UInt cvt_offsets[TA_SCRIPT_MAX];
+  FT_UInt cvt_horz_width_sizes[TA_SCRIPT_MAX];
+  FT_UInt cvt_vert_width_sizes[TA_SCRIPT_MAX];
+  FT_UInt cvt_blue_zone_sizes[TA_SCRIPT_MAX];
+  FT_UInt cvt_blue_adjustment_offsets[TA_SCRIPT_MAX];
 } glyf_Data;
 
 /* an SFNT table */
@@ -226,7 +237,7 @@ struct FONT_
   FT_Bool pre_hinting;
   FT_Bool hint_composites;
   FT_Bool ignore_restrictions;
-  FT_UInt fallback_script;
+  TA_Script fallback_script;
   FT_Bool symbol;
   FT_Bool dehint;
   FT_Bool debug;
@@ -301,7 +312,7 @@ TA_sfnt_create_glyf_data(SFNT* sfnt,
 FT_Error
 TA_sfnt_handle_coverage(SFNT* sfnt,
                         FONT* font);
-void
+FT_Bool
 TA_sfnt_adjust_master_coverage(SFNT* sfnt,
                                FONT* font);
 #if 0
