@@ -26,6 +26,7 @@
 
 #ifdef TA_DEBUG
 int _ta_debug = 0;
+int _ta_debug_global = 0;
 int _ta_debug_disable_horz_hints;
 int _ta_debug_disable_vert_hints;
 int _ta_debug_disable_blue_hints;
@@ -43,6 +44,7 @@ typedef struct Hints_Record_
 
 typedef struct Recorder_
 {
+  SFNT* sfnt;
   FONT* font;
   GLYPH* glyph; /* the current glyph */
   Hints_Record hints_record;
@@ -195,6 +197,12 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   TA_Segment seg;
   TA_Segment seg_limit;
 
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
+
+  FT_UInt script_id = data->script_ids
+                        [font->loader->metrics->script_class->script];
+
   FT_Outline outline = font->loader->gloader->base.outline;
 
   FT_UInt* args;
@@ -247,7 +255,7 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
   num_args = num_packed_segments
              + 2 * (num_segments - num_packed_segments)
              + 2 * recorder->num_wrap_around_segments
-             + 2;
+             + 3;
 
   /* collect all arguments temporarily in an array (in reverse order) */
   /* so that we can easily split into chunks of 255 args */
@@ -266,6 +274,8 @@ TA_sfnt_build_glyph_segments(SFNT* sfnt,
     *(arg--) = bci_create_segments_composite_0 + num_packed_segments;
   else
     *(arg--) = bci_create_segments_0 + num_packed_segments;
+
+  *(arg--) = CVT_SCALING_VALUE_OFFSET(script_id);
   *(arg--) = num_segments;
 
   base = 0;
@@ -903,7 +913,10 @@ TA_emit_hints_record(Recorder* recorder,
   endp = hints_record->buf + hints_record->buf_len;
   for (p = hints_record->buf; p < endp; p += 2)
     if (*p)
+    {
       need_words = 1;
+      break;
+    }
 
   /* with most fonts it is very rare */
   /* that any of the pushed arguments is larger than 0xFF, */
@@ -978,6 +991,8 @@ TA_emit_hints_records(Recorder* recorder,
 
   hints_record = hints_records;
 
+  /* emit hints records in `if' clauses, */
+  /* with the ppem size as the condition */
   for (i = 0; i < num_hints_records - 1; i++)
   {
     BCI(MPPEM);
@@ -1121,10 +1136,13 @@ TA_hints_recorder(TA_Action action,
   TA_Point points = hints->points;
 
   Recorder* recorder = (Recorder*)hints->user;
+  SFNT* sfnt = recorder->sfnt;
   FONT* font = recorder->font;
   FT_UShort* wraps = recorder->wrap_around_segments;
   FT_Byte* p = recorder->hints_record.buf;
 
+  FT_UInt script = font->loader->metrics->script_class->script;
+
   FT_UShort* ip;
   FT_UShort* limit;
 
@@ -1315,13 +1333,13 @@ TA_hints_recorder(TA_Action action,
 
       if (edge->best_blue_is_shoot)
       {
-        *(p++) = HIGH(CVT_BLUE_SHOOTS_OFFSET(font) + edge->best_blue_idx);
-        *(p++) = LOW(CVT_BLUE_SHOOTS_OFFSET(font) + edge->best_blue_idx);
+        *(p++) = HIGH(CVT_BLUE_SHOOTS_OFFSET(script) + edge->best_blue_idx);
+        *(p++) = LOW(CVT_BLUE_SHOOTS_OFFSET(script) + edge->best_blue_idx);
       }
       else
       {
-        *(p++) = HIGH(CVT_BLUE_REFS_OFFSET(font) + edge->best_blue_idx);
-        *(p++) = LOW(CVT_BLUE_REFS_OFFSET(font) + edge->best_blue_idx);
+        *(p++) = HIGH(CVT_BLUE_REFS_OFFSET(script) + edge->best_blue_idx);
+        *(p++) = LOW(CVT_BLUE_REFS_OFFSET(script) + edge->best_blue_idx);
       }
 
       *(p++) = HIGH(edge->first - segments);
@@ -1370,13 +1388,13 @@ TA_hints_recorder(TA_Action action,
 
       if (edge->best_blue_is_shoot)
       {
-        *(p++) = HIGH(CVT_BLUE_SHOOTS_OFFSET(font) + edge->best_blue_idx);
-        *(p++) = LOW(CVT_BLUE_SHOOTS_OFFSET(font) + edge->best_blue_idx);
+        *(p++) = HIGH(CVT_BLUE_SHOOTS_OFFSET(script) + edge->best_blue_idx);
+        *(p++) = LOW(CVT_BLUE_SHOOTS_OFFSET(script) + edge->best_blue_idx);
       }
       else
       {
-        *(p++) = HIGH(CVT_BLUE_REFS_OFFSET(font) + edge->best_blue_idx);
-        *(p++) = LOW(CVT_BLUE_REFS_OFFSET(font) + edge->best_blue_idx);
+        *(p++) = HIGH(CVT_BLUE_REFS_OFFSET(script) + edge->best_blue_idx);
+        *(p++) = LOW(CVT_BLUE_REFS_OFFSET(script) + edge->best_blue_idx);
       }
 
       *(p++) = HIGH(edge->first - segments);
@@ -1493,6 +1511,7 @@ TA_hints_recorder(TA_Action action,
 
 static FT_Error
 TA_init_recorder(Recorder* recorder,
+                 SFNT* sfnt,
                  FONT* font,
                  GLYPH* glyph,
                  TA_GlyphHints hints)
@@ -1509,6 +1528,7 @@ TA_init_recorder(Recorder* recorder,
   FT_UShort num_strong_points = 0;
   FT_UShort* wrap_around_segment;
 
+  recorder->sfnt = sfnt;
   recorder->font = font;
   recorder->glyph = glyph;
   recorder->num_segments = axis->num_segments;
@@ -1679,13 +1699,17 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   }
 
   /* computing the segments is resolution independent, */
-  /* thus the pixel size in this call is arbitrary */
-  error = FT_Set_Pixel_Sizes(face, 20, 20);
+  /* thus the pixel size in this call is arbitrary -- */
+  /* however, we avoid unnecessary debugging output */
+  /* if we use the lowest value of the hinting range */
+  error = FT_Set_Pixel_Sizes(face,
+                             font->hinting_range_min,
+                             font->hinting_range_min);
   if (error)
     return error;
 
 #ifdef TA_DEBUG
-  /* temporarily disable debugging output */
+  /* temporarily disable some debugging output */
   /* to avoid getting the information twice */
   _ta_debug_save = _ta_debug;
   _ta_debug = 0;
@@ -1740,7 +1764,8 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
   if (font->loader->metrics->script_class == &ta_dflt_script_class)
   {
     /* since `TA_init_recorder' hasn't been called yet, */
-    /* we manually initialize the `font' and `glyph' fields */
+    /* we manually initialize the `sfnt', `font', and `glyph' fields */
+    recorder.sfnt = sfnt;
     recorder.font = font;
     recorder.glyph = glyph;
 
@@ -1754,7 +1779,7 @@ TA_sfnt_build_glyph_instructions(SFNT* sfnt,
     goto Done1;
   }
 
-  error = TA_init_recorder(&recorder, font, glyph, hints);
+  error = TA_init_recorder(&recorder, sfnt, font, glyph, hints);
   if (error)
     goto Err;
 
