@@ -310,7 +310,9 @@
 #define sal_ref sal_best + 1
 #define sal_func sal_ref + 1
 #define sal_anchor sal_func + 1
-#define sal_point_min sal_anchor + 1
+#define sal_vwidth_data_offset sal_anchor + 1
+#define sal_scale sal_vwidth_data_offset + 1
+#define sal_point_min sal_scale + 1
 #define sal_point_max sal_point_min + 1
 #define sal_base sal_point_max + 1
 #define sal_num_packed_segments sal_base + 1
@@ -320,25 +322,29 @@
 /* bytecode function numbers */
 
 /* 0 */
-#define bci_round 0
+#define bci_align_top 0
+#define bci_round bci_align_top + 1
 #define bci_smooth_stem_width bci_round + 1
 #define bci_get_best_width bci_smooth_stem_width + 1
 #define bci_strong_stem_width bci_get_best_width + 1
 #define bci_loop_do bci_strong_stem_width + 1
 #define bci_loop bci_loop_do + 1
 #define bci_cvt_rescale bci_loop + 1
-#define bci_blue_round bci_cvt_rescale + 1
-#define bci_decrement_component_counter bci_blue_round + 1
+#define bci_cvt_rescale_range bci_cvt_rescale + 1
+#define bci_vwidth_data_store bci_cvt_rescale_range + 1
+#define bci_blue_round bci_vwidth_data_store + 1
+#define bci_blue_round_range bci_blue_round + 1
+#define bci_decrement_component_counter bci_blue_round_range + 1
 #define bci_get_point_extrema bci_decrement_component_counter + 1
 #define bci_nibbles bci_get_point_extrema + 1
 #define bci_number_set_is_element bci_nibbles + 1
 #define bci_number_set_is_element2 bci_number_set_is_element + 1
 
-/* 13 */
+/* 17 */
 #define bci_create_segment bci_number_set_is_element2 + 1
 #define bci_create_segments bci_create_segment + 1
 
-/* 15 */
+/* 19 */
 /* the next ten entries must stay in this order */
 #define bci_create_segments_0 bci_create_segments + 1
 #define bci_create_segments_1 bci_create_segments_0 + 1
@@ -353,7 +359,7 @@
 
 #define bci_create_segments_composite bci_create_segments_9 + 1
 
-/* 26 */
+/* 30 */
 /* the next ten entries must stay in this order */
 #define bci_create_segments_composite_0 bci_create_segments_composite + 1
 #define bci_create_segments_composite_1 bci_create_segments_composite_0 + 1
@@ -366,25 +372,25 @@
 #define bci_create_segments_composite_8 bci_create_segments_composite_7 + 1
 #define bci_create_segments_composite_9 bci_create_segments_composite_8 + 1
 
-/* 36 */
+/* 40 */
 #define bci_align_point bci_create_segments_composite_9 + 1
 #define bci_align_segment bci_align_point + 1
 #define bci_align_segments bci_align_segment + 1
 
-/* 39 */
+/* 43 */
 #define bci_scale_contour bci_align_segments + 1
 #define bci_scale_glyph bci_scale_contour + 1
 #define bci_scale_composite_glyph bci_scale_glyph + 1
 #define bci_shift_contour bci_scale_composite_glyph + 1
 #define bci_shift_subglyph bci_shift_contour + 1
 
-/* 44 */
+/* 48 */
 #define bci_ip_outer_align_point bci_shift_subglyph + 1
 #define bci_ip_on_align_points bci_ip_outer_align_point + 1
 #define bci_ip_between_align_point bci_ip_on_align_points + 1
 #define bci_ip_between_align_points bci_ip_between_align_point + 1
 
-/* 48 */
+/* 52 */
 #define bci_adjust_common bci_ip_between_align_points + 1
 #define bci_stem_common bci_adjust_common + 1
 #define bci_serif_common bci_stem_common + 1
@@ -392,12 +398,12 @@
 #define bci_serif_link1_common bci_serif_anchor_common + 1
 #define bci_serif_link2_common bci_serif_link1_common + 1
 
-/* 54 */
+/* 58 */
 #define bci_lower_bound bci_serif_link2_common + 1
 #define bci_upper_bound bci_lower_bound + 1
 #define bci_upper_lower_bound bci_upper_bound + 1
 
-/* 57 */
+/* 61 */
 #define bci_adjust_bound bci_upper_lower_bound + 1
 #define bci_stem_bound bci_adjust_bound + 1
 #define bci_link bci_stem_bound + 1
@@ -408,23 +414,23 @@
 /* the order of the `bci_action_*' entries must correspond */
 /* to the order of the TA_Action enumeration entries (in `tahints.h') */
 
-/* 63 */
+/* 67 */
 #define bci_action_ip_before bci_stem + 1
 #define bci_action_ip_after bci_action_ip_before + 1
 #define bci_action_ip_on bci_action_ip_after + 1
 #define bci_action_ip_between bci_action_ip_on + 1
 
-/* 67 */
+/* 71 */
 #define bci_action_blue bci_action_ip_between + 1
 #define bci_action_blue_anchor bci_action_blue + 1
 
-/* 69 */
+/* 73 */
 #define bci_action_anchor bci_action_blue_anchor + 1
 #define bci_action_anchor_serif bci_action_anchor + 1
 #define bci_action_anchor_round bci_action_anchor_serif + 1
 #define bci_action_anchor_round_serif bci_action_anchor_round + 1
 
-/* 73 */
+/* 77 */
 #define bci_action_adjust bci_action_anchor_round_serif + 1
 #define bci_action_adjust_serif bci_action_adjust + 1
 #define bci_action_adjust_round bci_action_adjust_serif + 1
@@ -434,13 +440,13 @@
 #define bci_action_adjust_bound_round bci_action_adjust_bound_serif + 1
 #define bci_action_adjust_bound_round_serif bci_action_adjust_bound_round + 1
 
-/* 81 */
+/* 85 */
 #define bci_action_link bci_action_adjust_bound_round_serif + 1
 #define bci_action_link_serif bci_action_link + 1
 #define bci_action_link_round bci_action_link_serif + 1
 #define bci_action_link_round_serif bci_action_link_round + 1
 
-/* 85 */
+/* 89 */
 #define bci_action_stem bci_action_link_round_serif + 1
 #define bci_action_stem_serif bci_action_stem + 1
 #define bci_action_stem_round bci_action_stem_serif + 1
@@ -450,31 +456,31 @@
 #define bci_action_stem_bound_round bci_action_stem_bound_serif + 1
 #define bci_action_stem_bound_round_serif bci_action_stem_bound_round + 1
 
-/* 93 */
+/* 97 */
 #define bci_action_serif bci_action_stem_bound_round_serif + 1
 #define bci_action_serif_lower_bound bci_action_serif + 1
 #define bci_action_serif_upper_bound bci_action_serif_lower_bound + 1
 #define bci_action_serif_upper_lower_bound bci_action_serif_upper_bound + 1
 
-/* 97 */
+/* 101 */
 #define bci_action_serif_anchor bci_action_serif_upper_lower_bound + 1
 #define bci_action_serif_anchor_lower_bound bci_action_serif_anchor + 1
 #define bci_action_serif_anchor_upper_bound bci_action_serif_anchor_lower_bound + 1
 #define bci_action_serif_anchor_upper_lower_bound bci_action_serif_anchor_upper_bound + 1
 
-/* 101 */
+/* 105 */
 #define bci_action_serif_link1 bci_action_serif_anchor_upper_lower_bound + 1
 #define bci_action_serif_link1_lower_bound bci_action_serif_link1 + 1
 #define bci_action_serif_link1_upper_bound bci_action_serif_link1_lower_bound + 1
 #define bci_action_serif_link1_upper_lower_bound bci_action_serif_link1_upper_bound + 1
 
-/* 105 */
+/* 109 */
 #define bci_action_serif_link2 bci_action_serif_link1_upper_lower_bound + 1
 #define bci_action_serif_link2_lower_bound bci_action_serif_link2 + 1
 #define bci_action_serif_link2_upper_bound bci_action_serif_link2_lower_bound + 1
 #define bci_action_serif_link2_upper_lower_bound bci_action_serif_link2_upper_bound + 1
 
-/* 109 */
+/* 113 */
 #define bci_hint_glyph bci_action_serif_link2_upper_lower_bound + 1
 
 #define NUM_FDEFS bci_hint_glyph + 1 /* must be last */
@@ -488,50 +494,103 @@
 
 #define cvtl_temp 0 /* used for creating twilight points */
 #define cvtl_0x10000 cvtl_temp + 1
-#define cvtl_scale cvtl_0x10000 + 1
-#define cvtl_funits_to_pixels cvtl_scale + 1
-#define cvtl_is_extra_light cvtl_funits_to_pixels + 1
-#define cvtl_is_subglyph cvtl_is_extra_light + 1
-#define cvtl_stem_width_function cvtl_is_subglyph + 1
+#define cvtl_funits_to_pixels cvtl_0x10000 + 1
+#define cvtl_is_subglyph cvtl_funits_to_pixels + 1
+#define cvtl_num_used_scripts cvtl_is_subglyph + 1
+#define cvtl_stem_width_function cvtl_num_used_scripts + 1
 #define cvtl_is_element cvtl_stem_width_function + 1
 
 #define cvtl_max_runtime cvtl_is_element + 1 /* must be last */
 
-/* symbolic names for compile-time CVT locations */
+
+/* symbolic names for build-time CVT locations */
 /* (assigned in `cvt') */
 
-/* the horizontal and vertical standard widths */
-#define CVT_HORZ_STANDARD_WIDTH_OFFSET(font) cvtl_max_runtime
-#define CVT_VERT_STANDARD_WIDTH_OFFSET(font) \
-          CVT_HORZ_STANDARD_WIDTH_OFFSET(font) + 1
+/* note that each script has its own set of CVT data, */
+/* to be accessed using the offsets in the `cvt_offsets' array */
+/* first script index is 0 */
 
-/* the horizontal stem widths */
-#define CVT_HORZ_WIDTHS_OFFSET(font) \
-          CVT_VERT_STANDARD_WIDTH_OFFSET(font) + 1
-#define CVT_HORZ_WIDTHS_SIZE(font) \
-          ((font->loader->hints.metrics->script_class->script == TA_SCRIPT_DFLT) \
-           ? 0 \
-           : ((TA_LatinMetrics)font->loader->hints.metrics)->axis[0].width_count)
+/* the following macros access the variables `font' and `sfnt' */
+#define CVT_DATA ((glyf_Data*)(font->tables[sfnt->glyf_idx].data))
 
-/* the vertical stem widths */
-#define CVT_VERT_WIDTHS_OFFSET(font) \
-          CVT_HORZ_WIDTHS_OFFSET(font) + CVT_HORZ_WIDTHS_SIZE(font)
-#define CVT_VERT_WIDTHS_SIZE(font) \
-          ((font->loader->hints.metrics->script_class->script == TA_SCRIPT_DFLT) \
-           ? 0 \
-           : ((TA_LatinMetrics)font->loader->hints.metrics)->axis[1].width_count)
+/*
+ * we have the following layout in the CVT table:
+ *
+ *   . values initialized at runtime (`cvtl_max_runtime' elements)
+ *   . scaling values for each script ID (`num_used_script' elements)    (*)
+ *   . offset to the vertical stem widths array for each script ID       (*)
+ *     (`num_used_script' elements)
+ *   . size of the vertical stem widths array for each script ID         (*)
+ *     (`num_used_script' elements)
+ *
+ *   script ID 0:
+ *     . horizontal standard width (1 element)
+ *     . horizontal stem widths (`cvt_horz_width_sizes[id_to_idx(0)]'
+ *       elements)
+ *     . vertical standard width (1 element)
+ *     . vertical stem widths (`cvt_vert_width_sizes[id_to_idx(0)]'
+ *       elements)
+ *     . flat blue zones (`cvt_blue_zone_sizes[id_to_idx(0)]' elements)
+ *     . round blue zones (`cvt_blue_zone_sizes[id_to_idx(0)]' elements)
+ *   script ID 1:
+ *     ...
+ *
+ * (*) see function `bci_create_segments' how these three arrays get
+ *     accessed
+ *
+ * note that the `id_to_idx' function is hypothetic since the code works
+ * exactly the opposite way: the `cvt_*' arrays are indexed by the script
+ * index, and the `script_ids' array maps script indices to script IDs
+ */
 
-/* the number of blue zones (including the artificial ones) */
-#define CVT_BLUES_SIZE(font) \
-          ((font->loader->hints.metrics->script_class->script == TA_SCRIPT_DFLT) \
-           ? 0 \
-           : ((TA_LatinMetrics)font->loader->hints.metrics)->axis[1].blue_count + 2)
+/* scaling value index of script ID id */
+#define CVT_SCALING_VALUE_OFFSET(id) \
+          cvtl_max_runtime + (id)
 
-/* the blue zone values for flat and round edges */
-#define CVT_BLUE_REFS_OFFSET(font) \
-          CVT_VERT_WIDTHS_OFFSET(font) + CVT_VERT_WIDTHS_SIZE(font)
-#define CVT_BLUE_SHOOTS_OFFSET(font) \
-          CVT_BLUE_REFS_OFFSET(font) + CVT_BLUES_SIZE(font)
+/* vwidth offset data of script ID id */
+#define CVT_VWIDTH_OFFSET_DATA(id) \
+          CVT_SCALING_VALUE_OFFSET(id) \
+          + CVT_DATA->num_used_scripts \
+
+/* vwidth size data of script ID id */
+#define CVT_VWIDTH_SIZE_DATA(id) \
+          CVT_VWIDTH_OFFSET_DATA(id) \
+          + CVT_DATA->num_used_scripts
+
+/* horizontal standard width indices of script i */
+#define CVT_HORZ_STANDARD_WIDTH_OFFSET(i) \
+          cvtl_max_runtime \
+          + 3 * CVT_DATA->num_used_scripts \
+          + CVT_DATA->cvt_offsets[i]
+/* start and size of horizontal stem widths array of script i */
+#define CVT_HORZ_WIDTHS_OFFSET(i) \
+          CVT_HORZ_STANDARD_WIDTH_OFFSET(i) + 1
+#define CVT_HORZ_WIDTHS_SIZE(i) \
+          CVT_DATA->cvt_horz_width_sizes[i]
+
+/* vertical standard width indices of script i */
+#define CVT_VERT_STANDARD_WIDTH_OFFSET(i) \
+          CVT_HORZ_WIDTHS_OFFSET(i) + CVT_HORZ_WIDTHS_SIZE(i)
+/* start and size of vertical stem widths array of script i */
+#define CVT_VERT_WIDTHS_OFFSET(i) \
+          CVT_VERT_STANDARD_WIDTH_OFFSET(i) + 1
+#define CVT_VERT_WIDTHS_SIZE(i) \
+          CVT_DATA->cvt_vert_width_sizes[i]
+
+/* number of blue zones (including artificial ones) of script i */
+#define CVT_BLUES_SIZE(i) \
+          CVT_DATA->cvt_blue_zone_sizes[i]
+
+/* start of blue zone arrays for flat and round edges of script i */
+#define CVT_BLUE_REFS_OFFSET(i) \
+          CVT_VERT_WIDTHS_OFFSET(i) + CVT_VERT_WIDTHS_SIZE(i)
+#define CVT_BLUE_SHOOTS_OFFSET(i) \
+          CVT_BLUE_REFS_OFFSET(i) + CVT_BLUES_SIZE(i)
+
+/* x height blue zone (shoot) index of script i (valid if < 0xFFFF) */
+#define CVT_X_HEIGHT_BLUE_OFFSET(i) \
+          CVT_BLUE_SHOOTS_OFFSET(i) \
+          + CVT_DATA->cvt_blue_adjustment_offsets[i]
 
 
 extern FT_Byte ttfautohint_glyph_bytecode[7];
