@@ -1207,6 +1207,14 @@ TA_sfnt_create_glyf_data(SFNT* sfnt,
 }
 
 
+/* While the auto-hinter is glyph oriented (this is, using `glyf' data), */
+/* it relies on the `cmap' table to get script coverage data. */
+/* In TTCs, subfonts normally share the same `glyf' table */
+/* but use different `cmap's.  Covering the most common situation, */
+/* namely a single `glyf' table and multiple `cmap's, */
+/* ttfautohint merges coverage data for the first subfont's `glyf' table */
+/* with all other subfonts that also use this very `glyf' table. */
+
 FT_Error
 TA_sfnt_handle_coverage(SFNT* sfnt,
                         FONT* font)
@@ -1219,7 +1227,7 @@ TA_sfnt_handle_coverage(SFNT* sfnt,
   FT_Face face = sfnt->face;
   TA_FaceGlobals curr_globals;
 
-  FT_UInt saved_fallback_script = font->fallback_script;
+  TA_Script saved_fallback_script = font->fallback_script;
 
 
   /* using TA_SCRIPT_NONE as the fallback script ensures */
@@ -1278,7 +1286,7 @@ Exit:
 }
 
 
-void
+FT_Bool
 TA_sfnt_adjust_master_coverage(SFNT* sfnt,
                                FONT* font)
 {
@@ -1306,7 +1314,10 @@ TA_sfnt_adjust_master_coverage(SFNT* sfnt,
         gscripts[nn] |= master_globals->font->fallback_script;
       }
     }
+    return 1; /* master coverage adjusted */
   }
+  else
+    return 0;
 }
 
 
