@@ -669,6 +669,12 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
 
       TA_Direction in_dir = TA_DIR_NONE;
 
+      FT_Pos last_good_in_x = 0;
+      FT_Pos last_good_in_y = 0;
+
+      FT_UInt units_per_em = hints->metrics->scaler.face->units_per_EM;
+      FT_Int near_limit = 20 * units_per_em / 2048;
+
 
       for (point = points; point < point_limit; point++)
       {
@@ -679,14 +685,58 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
         if (point == first)
         {
           prev = first->prev;
+
           in_x = first->fx - prev->fx;
           in_y = first->fy - prev->fy;
+
+          last_good_in_x = in_x;
+          last_good_in_y = in_y;
+
+          if (TA_ABS(in_x) + TA_ABS(in_y) < near_limit)
+          {
+            /* search first non-near point to get a good `in_dir' value */
+
+            TA_Point point_ = prev;
+
+
+            while (point_ != first)
+            {
+              TA_Point prev_ = point_->prev;
+
+              FT_Pos in_x_ = point_->fx - prev_->fx;
+              FT_Pos in_y_ = point_->fy - prev_->fy;
+
+
+              if (TA_ABS(in_x_) + TA_ABS(in_y_) >= near_limit)
+              {
+                last_good_in_x = in_x_;
+                last_good_in_y = in_y_;
+
+                break;
+              }
+
+              point_ = prev_;
+            }
+          }
+
           in_dir = ta_direction_compute(in_x, in_y);
           first = prev + 1;
         }
 
         point->in_dir = (FT_Char)in_dir;
 
+        /* check whether the current point is near to the previous one */
+        /* (value 20 in `near_limit' is heuristic; we use Taxicab */
+        /* metrics for the test) */
+
+        if (TA_ABS(in_x) + TA_ABS(in_y) < near_limit)
+          point->flags |= TA_FLAG_NEAR;
+        else
+        {
+          last_good_in_x = in_x;
+          last_good_in_y = in_y;
+        }
+
         next = point->next;
         out_x = next->fx - point->fx;
         out_y = next->fy - point->fy;
@@ -694,23 +744,44 @@ ta_glyph_hints_reload(TA_GlyphHints hints,
         in_dir = ta_direction_compute(out_x, out_y);
         point->out_dir = (FT_Char)in_dir;
 
-        /* check for weak points */
+        /* Check for weak points.  The remaining points not collected */
+        /* in edges are then implicitly classified as strong points. */
 
         if (point->flags & TA_FLAG_CONTROL)
         {
+          /* control points are always weak */
         Is_Weak_Point:
           point->flags |= TA_FLAG_WEAK_INTERPOLATION;
         }
         else if (point->out_dir == point->in_dir)
         {
           if (point->out_dir != TA_DIR_NONE)
+          {
+            /* current point lies on a horizontal or          */
+            /* vertical segment (but doesn't start or end it) */
             goto Is_Weak_Point;
+          }
 
-          if (ta_corner_is_flat(in_x, in_y, out_x, out_y))
+          /* test whether `in' and `out' direction is approximately */
+          /* the same (and use the last good `in' vector in case    */
+          /* the current point is near to the previous one)         */
+          if (ta_corner_is_flat(point->flags & TA_FLAG_NEAR ? last_good_in_x
+                                                            : in_x,
+                                point->flags & TA_FLAG_NEAR ? last_good_in_y
+                                                            : in_y,
+                                out_x,
+                                out_y))
+          {
+            /* current point lies on a straight, diagonal line */
+            /* (more or less)                                  */
             goto Is_Weak_Point;
+          }
         }
         else if (point->in_dir == -point->out_dir)
+        {
+          /* current point forms a spike */
           goto Is_Weak_Point;
+        }
 
         in_x = out_x;
         in_y = out_y;
