@@ -194,10 +194,17 @@ ta_loader_load_g(TA_Loader loader,
 
     /* now load the slot image into the auto-outline */
     /* and run the automatic hinting process */
-    if (metrics->clazz->script_hints_apply)
-      metrics->clazz->script_hints_apply(hints,
-                                         &gloader->current.outline,
-                                         metrics);
+    {
+      TA_WritingSystemClass writing_system_class =
+                              ta_writing_system_classes
+                                [metrics->script_class->writing_system];
+
+
+      if (writing_system_class->script_hints_apply)
+        writing_system_class->script_hints_apply(hints,
+                                                 &gloader->current.outline,
+                                                 metrics);
+    }
 
     /* we now need to adjust the metrics according to the change in */
     /* width/positioning that occurred during the hinting process */
@@ -529,23 +536,28 @@ ta_loader_load_glyph(FONT* font,
   if (!error)
   {
     TA_ScriptMetrics metrics;
-    FT_UInt options = 0;
+    FT_UInt options = TA_SCRIPT_DFLT;
 
 
 #ifdef FT_OPTION_AUTOFIT2
     /* XXX: undocumented hook to activate the latin2 hinter */
     if (load_flags & (1UL << 20))
-      options = 2;
+      options = TA_SCRIPT_LTN2;
 #endif
 
     error = ta_face_globals_get_metrics(loader->globals, gindex,
                                         options, &metrics);
     if (!error)
     {
+      TA_WritingSystemClass writing_system_class =
+                              ta_writing_system_classes
+                                [metrics->script_class->writing_system];
+
+
       loader->metrics = metrics;
 
-      if (metrics->clazz->script_metrics_scale)
-        metrics->clazz->script_metrics_scale(metrics, &scaler);
+      if (writing_system_class->script_metrics_scale)
+        writing_system_class->script_metrics_scale(metrics, &scaler);
       else
         metrics->scaler = scaler;
 
@@ -553,10 +565,10 @@ ta_loader_load_glyph(FONT* font,
                     | FT_LOAD_IGNORE_TRANSFORM;
       load_flags &= ~FT_LOAD_RENDER;
 
-      if (metrics->clazz->script_hints_init)
+      if (writing_system_class->script_hints_init)
       {
-        error = metrics->clazz->script_hints_init(&loader->hints,
-                                                  metrics);
+        error = writing_system_class->script_hints_init(&loader->hints,
+                                                        metrics);
         if (error)
           goto Exit;
       }
