@@ -72,6 +72,27 @@ unsigned char PREP(store_0x10000) [] =
 
 };
 
+/* store number of scripts (needed in `fpgm' table) */
+
+unsigned char PREP(store_num_used_scripts_a) [] =
+{
+
+  PUSHB_2,
+    cvtl_num_used_scripts,
+
+};
+
+/*  %c, number of used scripts */
+
+unsigned char PREP(store_num_used_scripts_b) [] =
+{
+
+  WCVTP,
+
+};
+
+/* if the current ppem value is an exception, don't apply scaling */
+
 unsigned char PREP(test_exception_a) [] =
 {
 
@@ -83,143 +104,81 @@ unsigned char PREP(test_exception_a) [] =
 
 };
 
+/* provide scaling factors for all scripts */
+
 unsigned char PREP(align_top_a) [] =
 {
 
-  /* optimize the alignment of the top of small letters to the pixel grid */
-
-  PUSHB_1,
+    PUSHB_2,
+      sal_i,
+      CVT_SCALING_VALUE_OFFSET(0),
+    WS,
 
 };
 
-/*  %c, index of alignment blue zone */
+/*  PUSH (num_used_scripts + 2) */
+/*    ... */
+/*    %c, script 1's x height blue zone idx */
+/*    %c, script 0's x height blue zone idx */
+/*    %c, num_used_scripts */
 
 unsigned char PREP(align_top_b) [] =
 {
 
-  RCVT,
-  DUP,
-  DUP,
-
-};
-
-unsigned char PREP(align_top_c1a) [] =
-{
-
-  /* use this if option `increase_x_height' > 0 */
-  /* apply much `stronger' rounding up of x height for */
-  /* 6 <= PPEM <= increase_x_height */
-  MPPEM,
-  PUSHW_1,
-
-};
-
-/*  %d, x height increase limit */
-
-unsigned char PREP(align_top_c1b) [] =
-{
-
-  LTEQ,
-  MPPEM,
-  PUSHB_1,
-    6,
-  GTEQ,
-  AND,
-  IF,
-    PUSHB_1,
-      52, /* threshold = 52 */
-
-  ELSE,
-    PUSHB_1,
-      40, /* threshold = 40 */
-
-  EIF,
-  ADD,
-  FLOOR, /* fitted = FLOOR(scaled + threshold) */
-
-};
-
-unsigned char PREP(align_top_c2) [] =
-{
-
-  PUSHB_1,
-    40,
-  ADD,
-  FLOOR, /* fitted = FLOOR(scaled + 40) */
-
-};
-
-unsigned char PREP(align_top_d) [] =
-{
-
-  DUP, /* s: scaled scaled fitted fitted */
-  ROLL,
-  NEQ,
-  IF, /* s: scaled fitted */
-    PUSHB_1,
-      2,
-    CINDEX,
-    SUB, /* s: scaled (fitted-scaled) */
-    PUSHB_1,
-      cvtl_0x10000,
-    RCVT,
-    MUL, /* (fitted-scaled) in 16.16 format */
-    SWAP,
-    DIV, /* ((fitted-scaled) / scaled) in 16.16 format */
-
-    PUSHB_1,
-      cvtl_scale,
-    SWAP,
-    WCVTP,
+      bci_align_top,
+    LOOPCALL,
 
 };
 
 unsigned char PREP(loop_cvt_a) [] =
 {
 
-    /* loop over vertical CVT entries */
-    PUSHB_3,
+    /* loop over (almost all) vertical CVT entries of all scripts, part 1 */
+    PUSHB_2,
+      sal_i,
+      CVT_SCALING_VALUE_OFFSET(0),
+    WS,
 
 };
 
-/*    %c, first vertical index */
-/*    %c, number of vertical indices */
+/*  PUSH (2*num_used_scripts + 2) */
+/*    ... */
+/*    %c, script 1's first vertical index */
+/*    %c, script 1's number of vertical indices */
+/*        (std. width, widths, flat blues zones without artifical ones) */
+/*    %c, script 0's first vertical index */
+/*    %c, script 0's number of vertical indices */
+/*        (std. width, widths, flat blues zones without artifical ones) */
+/*    %c, num_used_scripts */
 
 unsigned char PREP(loop_cvt_b) [] =
 {
 
-      bci_cvt_rescale,
+      bci_cvt_rescale_range,
     LOOPCALL,
 
-    /* loop over blue refs */
-    PUSHB_3,
+    /* loop over (almost all) vertical CVT entries of all scripts, part 2 */
+    PUSHB_2,
+      sal_i,
+      CVT_SCALING_VALUE_OFFSET(0),
+    WS,
 
 };
 
-/*    %c, first blue ref index */
-/*    %c, number of blue ref indices */
+/*  PUSH (2*num_used_scripts + 2) */
+/*    ... */
+/*    %c, script 1's first round blue zone index */
+/*    %c, script 1's number of round blue zones (without artificial ones) */
+/*    %c, script 0's first round blue zone index */
+/*    %c, script 0's number of round blue zones (without artificial ones) */
+/*    %c, num_used_scripts */
 
 unsigned char PREP(loop_cvt_c) [] =
 {
 
-      bci_cvt_rescale,
+      bci_cvt_rescale_range,
     LOOPCALL,
 
-    /* loop over blue shoots */
-    PUSHB_3,
-
-};
-
-/*    %c, first blue shoot index */
-/*    %c, number of blue shoot indices */
-
-unsigned char PREP(loop_cvt_d) [] =
-{
-
-      bci_cvt_rescale,
-    LOOPCALL,
-  EIF,
-
 };
 
 unsigned char PREP(test_exception_b) [] =
@@ -229,42 +188,75 @@ unsigned char PREP(test_exception_b) [] =
 
 };
 
-unsigned char PREP(compute_extra_light_a) [] =
+unsigned char PREP(store_vwidth_data_a) [] =
 {
 
-  /* compute (vertical) `extra_light' flag */
-  PUSHB_3,
-    cvtl_is_extra_light,
-    40,
+  PUSHB_2,
+    sal_i,
 
 };
 
-/*  %c, index of vertical standard_width */
+/*  %c, offset to vertical width offset data in CVT */
 
-unsigned char PREP(compute_extra_light_b) [] =
+unsigned char PREP(store_vwidth_data_b) [] =
 {
 
-  RCVT,
-  GT, /* standard_width < 40 */
-  WCVTP,
+  WS,
 
 };
 
-unsigned char PREP(round_blues_a) [] =
+/*PUSH (num_used_scripts + 2) */
+/*  ... */
+/*  %c, script 1's first vertical width index */
+/*  %c, script 0's first vertical width index */
+/*  %c, num_used_scripts */
+
+unsigned char PREP(store_vwidth_data_c) [] =
 {
 
-  /* use discrete values for blue zone widths */
-  PUSHB_3,
+    bci_vwidth_data_store,
+  LOOPCALL,
+
+  PUSHB_2,
+    sal_i,
 
 };
 
-/*  %c, first blue ref index */
-/*  %c, number of blue ref indices */
+/*  %c, offset to vertical width size data in CVT */
 
-unsigned char PREP(round_blues_b) [] =
+unsigned char PREP(store_vwidth_data_d) [] =
 {
 
-    bci_blue_round,
+  WS,
+
+};
+
+/*PUSH (num_used_scripts + 2) */
+/*  ... */
+/*  %c, script 1's number of vertical widths */
+/*  %c, script 0's number of vertical widths */
+/*  %c, num_used_scripts */
+
+unsigned char PREP(store_vwidth_data_e) [] =
+{
+
+    bci_vwidth_data_store,
+  LOOPCALL,
+
+};
+
+/*PUSH (2*num_used_scripts + 2) */
+/*  ... */
+/*  %c, script 1's first blue ref index */
+/*  %c, script 1's number of blue ref indices */
+/*  %c, script 0's first blue ref index */
+/*  %c, script 0's number of blue ref indices */
+/*  %c, num_used_scripts */
+
+unsigned char PREP(round_blues) [] =
+{
+
+    bci_blue_round_range,
   LOOPCALL,
 
 };
@@ -337,11 +329,20 @@ unsigned char PREP(set_stem_width_handling_c) [] =
       /* version >= 38, sub-pixel positioning is enabled */
       LTEQ,
       IF,
-        /* check whether sub-pixel positioning is enabled (bit 10) */
-        PUSHW_1,
-          0x04,
-          0x00,
+        /* check whether sub-pixel positioning is enabled (bit 10) -- */
+        /* due to a bug in FreeType 2.5.0 and earlier, */
+        /* bit 6 must be set also to get the correct information, */
+        /* so we test that both return values (in bits 13 and 17) are set */
+        PUSHW_3,
+          0x08, /* bits 13 and 17 shifted by 6 bits */
+          0x80,
+          0x00, /* we do `MUL' with value 1, */
+          0x01, /* which is essentially a division by 64 */
+          0x04, /* bits 6 and 10 */
+          0x40,
         GETINFO,
+        MUL,
+        EQ,
         IF,
           PUSHB_2,
             cvtl_stem_width_function,
@@ -551,9 +552,9 @@ Fail:
 #define COPY_PREP(snippet_name) \
           do \
           { \
-            memcpy(buf_p, prep_ ## snippet_name, \
+            memcpy(bufp, prep_ ## snippet_name, \
                    sizeof (prep_ ## snippet_name)); \
-            buf_p += sizeof (prep_ ## snippet_name); \
+            bufp += sizeof (prep_ ## snippet_name); \
           } while (0)
 
 static FT_Error
@@ -562,9 +563,10 @@ TA_table_build_prep(FT_Byte** prep,
                     SFNT* sfnt,
                     FONT* font)
 {
-  TA_LatinAxis vaxis;
-  TA_LatinBlue blue_adjustment = NULL;
-  FT_UInt i;
+  SFNT_Table* glyf_table = &font->tables[sfnt->glyf_idx];
+  glyf_Data* data = (glyf_Data*)glyf_table->data;
+
+  FT_Int i;
 
   FT_Byte* buf = NULL;
   FT_Byte* buf_new;
@@ -572,34 +574,18 @@ TA_table_build_prep(FT_Byte** prep,
   FT_UInt buf_new_len;
 
   FT_UInt len;
-  FT_Byte* buf_p = NULL;
+  FT_Byte* bufp = NULL;
 
 
-  if (font->loader->hints.metrics->script_class->script == TA_SCRIPT_NONE)
-    vaxis = NULL;
-  else
+  if (font->x_height_snapping_exceptions)
   {
-    vaxis = &((TA_LatinMetrics)font->loader->hints.metrics)->axis[1];
-
-    for (i = 0; i < vaxis->blue_count; i++)
-    {
-      if (vaxis->blues[i].flags & TA_LATIN_BLUE_ADJUSTMENT)
-      {
-        blue_adjustment = &vaxis->blues[i];
-        break;
-      }
-    }
-  }
-
-  if (blue_adjustment && font->x_height_snapping_exceptions)
-  {
-    buf_p = TA_sfnt_build_number_set(sfnt, &buf,
-                                     font->x_height_snapping_exceptions);
-    if (!buf_p)
+    bufp = TA_sfnt_build_number_set(sfnt, &buf,
+                                    font->x_height_snapping_exceptions);
+    if (!bufp)
       return FT_Err_Out_Of_Memory;
   }
 
-  buf_len = buf_p - buf;
+  buf_len = bufp - buf;
   buf_new_len = buf_len;
 
   if (font->hinting_limit)
@@ -608,40 +594,48 @@ TA_table_build_prep(FT_Byte** prep,
                    + sizeof (PREP(hinting_limit_b));
 
   buf_new_len += sizeof (PREP(store_0x10000));
-
-  if (blue_adjustment)
-  {
-    if (font->x_height_snapping_exceptions)
-      buf_new_len += sizeof (PREP(test_exception_a));
-    buf_new_len += sizeof (PREP(align_top_a))
-                   + 1
-                   + sizeof (PREP(align_top_b))
-                   + (font->increase_x_height
-                       ? (sizeof (PREP(align_top_c1a))
-                          + 2
-                          + sizeof (PREP(align_top_c1b)))
-                       : sizeof (PREP(align_top_c2)))
-                   + sizeof (PREP(align_top_d))
-                   + sizeof (PREP(loop_cvt_a))
-                   + 2
-                   + sizeof (PREP(loop_cvt_b))
-                   + 2
-                   + sizeof (PREP(loop_cvt_c))
-                   + 2
-                   + sizeof (PREP(loop_cvt_d));
-    if (font->x_height_snapping_exceptions)
-      buf_new_len += sizeof (PREP(test_exception_b));
-  }
-
-  buf_new_len += sizeof (PREP(compute_extra_light_a))
+  buf_new_len += sizeof (PREP(store_num_used_scripts_a))
                  + 1
-                 + sizeof (PREP(compute_extra_light_b));
+                 + sizeof (PREP(store_num_used_scripts_b));
 
-  if (CVT_BLUES_SIZE(font))
-    buf_new_len += sizeof (PREP(round_blues_a))
-                   + 2
-                   + sizeof (PREP(round_blues_b));
+  if (font->x_height_snapping_exceptions)
+    buf_new_len += sizeof (PREP(test_exception_a));
 
+  buf_new_len += sizeof (PREP(align_top_a))
+                 + (data->num_used_scripts > 6
+                      ? data->num_used_scripts + 3
+                      : data->num_used_scripts + 2)
+                 + sizeof (PREP(align_top_b));
+  buf_new_len += sizeof (PREP(loop_cvt_a))
+                 + (data->num_used_scripts > 3
+                     ? 2 * data->num_used_scripts + 3
+                     : 2 * data->num_used_scripts + 2)
+                 + sizeof (PREP(loop_cvt_b))
+                 + (data->num_used_scripts > 3
+                     ? 2 * data->num_used_scripts + 3
+                     : 2 * data->num_used_scripts + 2)
+                 + sizeof (PREP(loop_cvt_c));
+
+  if (font->x_height_snapping_exceptions)
+    buf_new_len += sizeof (PREP(test_exception_b));
+
+  buf_new_len += sizeof (PREP(store_vwidth_data_a))
+                 + 1
+                 + sizeof (PREP(store_vwidth_data_b))
+                 + (data->num_used_scripts > 6
+                     ? data->num_used_scripts + 3
+                     : data->num_used_scripts + 2)
+                 + sizeof (PREP(store_vwidth_data_c))
+                 + 1
+                 + sizeof (PREP(store_vwidth_data_d))
+                 + (data->num_used_scripts > 6
+                     ? data->num_used_scripts + 3
+                     : data->num_used_scripts + 2)
+                 + sizeof (PREP(store_vwidth_data_e));
+  buf_new_len += (data->num_used_scripts > 3
+                     ? 2 * data->num_used_scripts + 3
+                     : 2 * data->num_used_scripts + 2)
+                 + sizeof (PREP(round_blues));
   buf_new_len += sizeof (PREP(set_stem_width_handling_a))
                  + 1
                  + sizeof (PREP(set_stem_width_handling_b))
@@ -669,75 +663,159 @@ TA_table_build_prep(FT_Byte** prep,
 
   /* copy remaining cvt program into buffer */
   /* and fill in the missing variables */
-  buf_p = buf + buf_len;
+  bufp = buf + buf_len;
 
   if (font->hinting_limit)
   {
     COPY_PREP(hinting_limit_a);
-    *(buf_p++) = HIGH(font->hinting_limit);
-    *(buf_p++) = LOW(font->hinting_limit);
+    *(bufp++) = HIGH(font->hinting_limit);
+    *(bufp++) = LOW(font->hinting_limit);
     COPY_PREP(hinting_limit_b);
   }
 
   COPY_PREP(store_0x10000);
 
-  if (blue_adjustment)
+  COPY_PREP(store_num_used_scripts_a);
+  *(bufp++) = (unsigned char)data->num_used_scripts;
+  COPY_PREP(store_num_used_scripts_b);
+
+  if (font->x_height_snapping_exceptions)
+    COPY_PREP(test_exception_a);
+
+  COPY_PREP(align_top_a);
+  if (data->num_used_scripts > 6)
   {
-    if (font->x_height_snapping_exceptions)
-      COPY_PREP(test_exception_a);
-
-    COPY_PREP(align_top_a);
-    *(buf_p++) = (unsigned char)(CVT_BLUE_SHOOTS_OFFSET(font)
-                                 + blue_adjustment - vaxis->blues);
-    COPY_PREP(align_top_b);
-    if (font->increase_x_height)
-    {
-      COPY_PREP(align_top_c1a);
-      *(buf_p++) = HIGH(font->increase_x_height);
-      *(buf_p++) = LOW(font->increase_x_height);
-      COPY_PREP(align_top_c1b);
-    }
-    else
-      COPY_PREP(align_top_c2);
-    COPY_PREP(align_top_d);
-
-    COPY_PREP(loop_cvt_a);
-    *(buf_p++) = (unsigned char)CVT_VERT_WIDTHS_OFFSET(font);
-    *(buf_p++) = (unsigned char)CVT_VERT_WIDTHS_SIZE(font);
-    /* don't loop over the artificial blue zones */
-    COPY_PREP(loop_cvt_b);
-    *(buf_p++) = (unsigned char)CVT_BLUE_REFS_OFFSET(font);
-    *(buf_p++) = (unsigned char)(CVT_BLUES_SIZE(font) - 2);
-    COPY_PREP(loop_cvt_c);
-    *(buf_p++) = (unsigned char)CVT_BLUE_SHOOTS_OFFSET(font);
-    *(buf_p++) = (unsigned char)(CVT_BLUES_SIZE(font) - 2);
-    COPY_PREP(loop_cvt_d);
-
-    if (font->x_height_snapping_exceptions)
-      COPY_PREP(test_exception_b);
+    BCI(NPUSHB);
+    BCI(data->num_used_scripts + 2);
   }
-
-  COPY_PREP(compute_extra_light_a);
-  *(buf_p++) = (unsigned char)CVT_VERT_STANDARD_WIDTH_OFFSET(font);
-  COPY_PREP(compute_extra_light_b);
-
-  if (CVT_BLUES_SIZE(font))
+  else
+    BCI(PUSHB_1 - 1 + data->num_used_scripts + 2);
+  /* XXX: make this work for offsets > 255 */
+  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
   {
-    COPY_PREP(round_blues_a);
-    *(buf_p++) = (unsigned char)CVT_BLUE_REFS_OFFSET(font);
-    *(buf_p++) = (unsigned char)CVT_BLUES_SIZE(font);
-    COPY_PREP(round_blues_b);
+    if (data->script_ids[i] == 0xFFFFU)
+      continue;
+
+    *(bufp++) = CVT_X_HEIGHT_BLUE_OFFSET(i) >= 0xFFFFU
+                  ? 0
+                  : (unsigned char)CVT_X_HEIGHT_BLUE_OFFSET(i);
   }
+  *(bufp++) = data->num_used_scripts;
+  COPY_PREP(align_top_b);
+
+  COPY_PREP(loop_cvt_a);
+  if (data->num_used_scripts > 3)
+  {
+    BCI(NPUSHB);
+    BCI(2 * data->num_used_scripts + 2);
+  }
+  else
+    BCI(PUSHB_1 - 1 + 2 * data->num_used_scripts + 2);
+  /* XXX: make this work for offsets > 255 */
+  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  {
+    if (data->script_ids[i] == 0xFFFFU)
+      continue;
+
+    /* don't loop over artificial blue zones */
+    *(bufp++) = (unsigned char)CVT_VERT_STANDARD_WIDTH_OFFSET(i);
+    *(bufp++) = (unsigned char)(1
+                                + CVT_VERT_WIDTHS_SIZE(i)
+                                + CVT_BLUES_SIZE(i) - 2);
+  }
+  *(bufp++) = data->num_used_scripts;
+  COPY_PREP(loop_cvt_b);
+  if (data->num_used_scripts > 3)
+  {
+    BCI(NPUSHB);
+    BCI(2 * data->num_used_scripts + 2);
+  }
+  else
+    BCI(PUSHB_1 - 1 + 2 * data->num_used_scripts + 2);
+  /* XXX: make this work for offsets > 255 */
+  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  {
+    if (data->script_ids[i] == 0xFFFFU)
+      continue;
+
+    /* don't loop over artificial blue zones */
+    *(bufp++) = (unsigned char)CVT_BLUE_SHOOTS_OFFSET(i);
+    *(bufp++) = (unsigned char)(CVT_BLUES_SIZE(i) - 2);
+  }
+  *(bufp++) = data->num_used_scripts;
+  COPY_PREP(loop_cvt_c);
+
+  if (font->x_height_snapping_exceptions)
+    COPY_PREP(test_exception_b);
+
+  COPY_PREP(store_vwidth_data_a);
+  *(bufp++) = (unsigned char)CVT_VWIDTH_OFFSET_DATA(0);
+  COPY_PREP(store_vwidth_data_b);
+  if (data->num_used_scripts > 6)
+  {
+    BCI(NPUSHB);
+    BCI(data->num_used_scripts + 2);
+  }
+  else
+    BCI(PUSHB_1 - 1 + data->num_used_scripts + 2);
+  /* XXX: make this work for offsets > 255 */
+  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  {
+    if (data->script_ids[i] == 0xFFFFU)
+      continue;
+
+    *(bufp++) = (unsigned char)CVT_VERT_WIDTHS_OFFSET(i);
+  }
+  *(bufp++) = data->num_used_scripts;
+  COPY_PREP(store_vwidth_data_c);
+  *(bufp++) = (unsigned char)CVT_VWIDTH_SIZE_DATA(0);
+  COPY_PREP(store_vwidth_data_d);
+  if (data->num_used_scripts > 6)
+  {
+    BCI(NPUSHB);
+    BCI(data->num_used_scripts + 2);
+  }
+  else
+    BCI(PUSHB_1 - 1 + data->num_used_scripts + 2);
+  /* XXX: make this work for values > 255 */
+  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  {
+    if (data->script_ids[i] == 0xFFFFU)
+      continue;
+
+    *(bufp++) = (unsigned char)CVT_VERT_WIDTHS_SIZE(i);
+  }
+  *(bufp++) = data->num_used_scripts;
+  COPY_PREP(store_vwidth_data_e);
+
+  if (data->num_used_scripts > 3)
+  {
+    BCI(NPUSHB);
+    BCI(2 * data->num_used_scripts + 2);
+  }
+  else
+    BCI(PUSHB_1 - 1 + 2 * data->num_used_scripts + 2);
+  /* XXX: make this work for offsets > 255 */
+  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  {
+    if (data->script_ids[i] == 0xFFFFU)
+      continue;
+
+    *(bufp++) = (unsigned char)CVT_BLUE_REFS_OFFSET(i);
+    *(bufp++) = (unsigned char)CVT_BLUES_SIZE(i);
+  }
+  *(bufp++) = data->num_used_scripts;
+  COPY_PREP(round_blues);
 
   COPY_PREP(set_stem_width_handling_a);
-  *(buf_p++) = font->gray_strong_stem_width ? bci_strong_stem_width
-                                            : bci_smooth_stem_width;
+  *(bufp++) = font->gray_strong_stem_width ? bci_strong_stem_width
+                                           : bci_smooth_stem_width;
   COPY_PREP(set_stem_width_handling_b);
-  *(buf_p++) = font->gdi_cleartype_strong_stem_width ? bci_strong_stem_width
-                                                     : bci_smooth_stem_width;
-  COPY_PREP(set_stem_width_handling_c);
-  *(buf_p++) = font->dw_cleartype_strong_stem_width ? bci_strong_stem_width
+  *(bufp++) = font->gdi_cleartype_strong_stem_width ? bci_strong_stem_width
                                                     : bci_smooth_stem_width;
+  COPY_PREP(set_stem_width_handling_c);
+  *(bufp++) = font->dw_cleartype_strong_stem_width ? bci_strong_stem_width
+                                                   : bci_smooth_stem_width;
   COPY_PREP(set_stem_width_handling_d);
   COPY_PREP(set_dropout_mode);
   COPY_PREP(reset_component_counter);
