@@ -25,6 +25,8 @@
 #include FT_FREETYPE_H
 #include FT_OUTLINE_H
 
+#include "tablue.h"
+
 
 /* enable one of the following three definitions for debugging */
 /* #define TA_DEBUG */
@@ -53,11 +55,19 @@
       _ta_message x; \
   } while (0)
 
+#define TA_LOG_GLOBAL(x) \
+  do \
+  { \
+    if (_ta_debug_global) \
+      _ta_message x; \
+  } while (0)
+
 void
 _ta_message(const char* format,
             ...);
 
 extern int _ta_debug;
+extern int _ta_debug_global;
 extern int _ta_debug_disable_horz_hints;
 extern int _ta_debug_disable_vert_hints;
 extern int _ta_debug_disable_blue_hints;
@@ -68,6 +78,9 @@ extern void* _ta_debug_hints;
 #define TA_LOG(x) \
   do { } while (0) /* nothing */
 
+#define TA_LOG_GLOBAL(x) \
+  do { } while (0) /* nothing */
+
 #endif /* !TA_DEBUG */
 
 
@@ -239,14 +252,13 @@ typedef struct TA_WritingSystemClassRec_
  */
 
 #undef SCRIPT
-#define SCRIPT(s, S) \
+#define SCRIPT(s, S, d) \
           TA_SCRIPT_ ## S,
 
 /* The list of known scripts. */
 typedef enum TA_Script_
 {
-
-#include "tascript.h"
+#include <ttfautohint-scripts.h>
 
   TA_SCRIPT_MAX /* do not remove */
 } TA_Script;
@@ -266,6 +278,7 @@ typedef const TA_Script_UniRangeRec* TA_Script_UniRange;
 typedef struct TA_ScriptClassRec_
 {
   TA_Script script;
+  TA_Blue_Stringset blue_stringset;
   TA_WritingSystem writing_system;
 
   TA_Script_UniRange script_uni_ranges; /* last must be { 0, 0 } */
